@@ -1,6 +1,6 @@
 package Language::Expr::Parser;
 BEGIN {
-  $Language::Expr::Parser::VERSION = '0.16';
+  $Language::Expr::Parser::VERSION = '0.17';
 }
 # ABSTRACT: Parse Language::Expr expression
 
@@ -39,83 +39,177 @@ sub parse_expr {
         <rule: answer>
             <MATCH=or_xor>
 
-# precedence level  2: left     =>
+# precedence level: left     =>
         <rule: pair>
-            <key=(\w+)> =\> <value=answer>
+            <key=(\w++)> =\> <value=answer>
             (?{ $MATCH = $obj->rule_pair_simple(match=>\%MATCH) })
           | <key=squotestr> =\> <value=answer>
             (?{ $MATCH = $obj->rule_pair_string(match=>\%MATCH) })
           | <key=dquotestr> =\> <value=answer>
             (?{ $MATCH = $obj->rule_pair_string(match=>\%MATCH) })
 
-# precedence level  3: left     || // ^^
+# precedence level: left     || // ^^
         <rule: or_xor>
-            <[operand=and]> ** <[op=(\|\||//|\^\^)]>
-            (?{ $MATCH = $obj->rule_or_xor(match=>\%MATCH) })
+            <[operand=ternary]> ** <[op=(\|\||//|\^\^)]>
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_or_xor(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level  4: left     &&
+# precedence level: right    ?:
+        <rule: ternary>
+            <[operand=and]> ** <[op=(\?|:)]>
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    unless (@{ $MATCH{op} } == 2 &&
+                            $MATCH{op}[0] eq '?' &&
+                            $MATCH{op}[1] eq ':') {
+                        die "Invalid syntax for ternary, please use X ? Y : Z syntax";
+                    }
+                    $MATCH = $obj->rule_ternary(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
+
+# precedence level: left     &&
         <rule: and>
             <[operand=bit_or_xor]> ** <[op=(&&)]>
-            (?{ $MATCH = $obj->rule_and(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_and(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level  5: left     | ^
+# precedence level: left     | ^
         <rule: bit_or_xor>
             <[operand=bit_and]> ** <[op=(\||\^)]>
-            (?{ $MATCH = $obj->rule_bit_or_xor(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_bit_or_xor(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level  6: left     &
+# precedence level: left     &
         <rule: bit_and>
             <[operand=comparison3]> ** <[op=(&)]>
-            (?{ $MATCH = $obj->rule_bit_and(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_bit_and(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
             # NOTE: \x3c = "<", \x3e = ">"
 
-# precedence level  7: nonassoc (currently the grammar says assoc) <=> cmp
+# precedence level: nonassoc (currently the grammar says assoc) <=> cmp
         <rule: comparison3>
             <[operand=comparison]> ** <[op=(\x3c=\x3e|cmp)]>
-            (?{ $MATCH = $obj->rule_comparison3(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_comparison3(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level  8: left == != eq ne < > <= >= ge gt le lt
+# precedence level: left == != eq ne < > <= >= ge gt le lt
         <rule: comparison>
             <[operand=bit_shift]> ** <[op=(==|!=|eq|ne|\x3c=?|\x3e=?|lt|gt|le|ge)]>
-            (?{ $MATCH = $obj->rule_comparison(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_comparison(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level  9: left     << >>
+# precedence level: left     << >>
         <rule: bit_shift>
             <[operand=add]> ** <[op=(\x3c\x3c|\x3e\x3e)]>
-            (?{ $MATCH = $obj->rule_bit_shift(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_bit_shift(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level 10: left     + - .
+# precedence level: left     + - .
         <rule: add>
             <[operand=mult]> ** <[op=(\+|-|\.)]>
-            (?{ $MATCH = $obj->rule_add(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_add(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level 11: left     * / % x
+# precedence level: left     * / % x
         <rule: mult>
             <[operand=unary]> ** <[op=(\*|/|%|x)]>
-            (?{ $MATCH = $obj->rule_mult(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_mult(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level 12: right    ! ~ unary+ unary-
+# precedence level: right    ! ~ unary+ unary-
         <rule: unary>
             <[op=(!|~|\+|-)]>* <operand=power>
-            (?{ $MATCH = $obj->rule_unary(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_unary(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand};
+                }
+            })
 
-# precedence level 13: right    **
+# precedence level: right    **
         <rule: power>
             <[operand=subscripting]> ** <[op=(\*\*)]>
-            (?{ $MATCH = $obj->rule_power(match=>\%MATCH) })
+            (?{
+                if ($MATCH{op} && @{ $MATCH{op} }) {
+                    $MATCH = $obj->rule_power(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand}[0];
+                }
+            })
 
-# precedence level 14: left    hash[s], array[i]
+# precedence level: left    hash[s], array[i]
         <rule: subscripting>
             <operand=var0> <[subscript]>*
-            (?{ $MATCH = $obj->rule_subscripting_var(match=>\%MATCH) })
+            (?{
+                if ($MATCH{subscript} && @{ $MATCH{subscript} }) {
+                    $MATCH = $obj->rule_subscripting_var(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand};
+                }
+            })
           | <operand=term> <[subscript]>*
-            (?{ $MATCH = $obj->rule_subscripting_expr(match=>\%MATCH) })
+            (?{
+                if ($MATCH{subscript} && @{ $MATCH{subscript} }) {
+                    $MATCH = $obj->rule_subscripting_expr(match=>\%MATCH);
+                } else {
+                    $MATCH = $MATCH{operand};
+                }
+            })
 
         <rule: subscript>
               \[ <MATCH=term> \]
 
-# precedence level 15: left     term (variable, str/num literals, func(), (paren))
+# precedence level: left     term (variable, str/num literals, func(), (paren))
         <rule: term>
             <MATCH=func>
           | <MATCH=var0>
@@ -149,13 +243,13 @@ sub parse_expr {
             (?{ $MATCH = $obj->rule_bool(match=>\%MATCH) })
 
         <token: num0>
-            <sign0a=([+-]?)> 0x <num0a=([0-9A-Fa-f]++)>
+            <sign0a=([+-]?+)> 0x <num0a=([0-9A-Fa-f]++)>
             (?{ $MATCH = $obj->rule_num(match=>{num=>
                 ($MATCH{sign0a} eq '-' ? -1:1) * hex($MATCH{num0a})}) })
-          | <sign0b=([+-]?)> 0o <num0b=([0-7]++)>
+          | <sign0b=([+-]?+)> 0o <num0b=([0-7]++)>
             (?{ $MATCH = $obj->rule_num(match=>{num=>
                 ($MATCH{sign0b} eq '-' ? -1:1) * oct($MATCH{num0b})}) })
-          | <sign0c=([+-]?)> 0b <num0c=([0-1]++)>
+          | <sign0c=([+-]?+)> 0b <num0c=([0-1]++)>
             (?{ $MATCH = $obj->rule_num(match=>{num=>
                 ($MATCH{sign0c} eq '-' ? -1:1) * oct("0b".$MATCH{num0c})}) })
           | <num0c=( [+-]?\d++(?:\.\d++)?+ | inf | nan)>
@@ -166,26 +260,26 @@ sub parse_expr {
           | <MATCH=dquotestr>
 
         <token: squotestr>
-            '<[part=(\\\\|\\'|\\|[^\\']+)]>*'
+            '<[part=(\\\\|\\'|\\|[^\\']++)]>*'
             (?{ $MATCH = $obj->rule_squotestr(match=>\%MATCH) })
 
         <token: dquotestr>
-            "<[part=([^"\044\\]+|\$\.\.?|\$\w+|\$\{[^\}]+\}|\\\\|\\'|\\"|\\[tnrfbae\$]|\\[0-7]{1,3}|\\x[0-9A-Fa-f]{1,2}|\\x\{[0-9A-Fa-f]{1,4}\}|\\)]>*"
+            "<[part=([^"\044\\]++|\$\.\.?|\$\w+|\$\{[^\}]+\}|\\\\|\\'|\\"|\\[tnrfbae\$]|\\[0-7]{1,3}|\\x[0-9A-Fa-f]{1,2}|\\x\{[0-9A-Fa-f]{1,4}\}|\\)]>*"
             (?{ $MATCH = $obj->rule_dquotestr(match=>\%MATCH) })
 
         <rule: var0>
-            \$ <var=(\w+(?:::\w+)*)>
+            \$ <var=(\w++(?:::\w+)*+)>
             (?{ $MATCH = $obj->rule_var(match=>\%MATCH) })
-          | \$ \{ <var=([^\}]+)> \}
+          | \$ \{ <var=([^\}]++)> \}
             (?{ $MATCH = $obj->rule_var(match=>\%MATCH) })
 
         <rule: func>
-            <func_name=([A-Za-z_]\w*)> \( \)
+            <func_name=([A-Za-z_]\w*+)> \( \)
             (?{ $MATCH = $obj->rule_func(match=>{func_name=>$MATCH{func_name}, args=>[]}) })
           | <func_name=(map|grep|usort)> \( \{ <expr=answer> \} (?{ push @$subexpr_stack, $CONTEXT }), <input_array=answer> \)
             (?{ my $meth = "rule_func_$MATCH{func_name}";
                 $MATCH = $obj->$meth(match=>{expr=>pop(@$subexpr_stack), array=>$MATCH{input_array}}) })
-          | <func_name=([A-Za-z_]\w*)> \( <[args=answer]> ** (,) \)
+          | <func_name=([A-Za-z_]\w*+)> \( <[args=answer]> ** (,) \)
             (?{ $MATCH = $obj->rule_func(match=>\%MATCH) })
 
     }xms } 0..($MAX_LEVELS-1)];
@@ -197,6 +291,7 @@ sub parse_expr {
     $obj_arg->expr_postprocess(result => $/{answer});
 }
 
+
 1;
 
 __END__
@@ -208,7 +303,7 @@ Language::Expr::Parser - Parse Language::Expr expression
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 METHODS
 
@@ -216,6 +311,14 @@ version 0.16
 
 Parse expression in $str. Will call various rule_*() methods in $obj.
 
+=head1 BUGS
+
+=over 4
+
+=item * Ternary operator is not chainable yet.
+
+=back
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
